def project_id(project_id):
    if (project_id==False):
        raise Exception("Define a project_id")
    
def job_id(job_id):
    if (job_id==False):
        raise Exception("Define a job_id")
    
def config(config):
    if config==False:
        raise Exception("Define a configuration")

